#include "TLS3008.h"

// TLS3008 library Version 1.0.0

TLS3008::TLS3008( byte bPin ) {	// Constructor
	this->_bLEDPin=13;
	this->_bPulsUs=2; // Pulse length
	this->_bEOSMs=35; // End Of Sync delay in ms
	this->_bEORMs=2; // End Of Reset delay in ms
	this->_bEOEFrameMs=3; // End of Empty Data Frame delay in ms
	this->_bEOFrameMs=5; // End of Data Frame delay (end of all leds data in string)
	this->_bFrameCount=9999; // Sent Frame counter
	setOutputPin( bPin ); // Setup output pin
//	initCom(); // Initialize communication
}

/*
TLS3008::~TLS3008() {	// Destructor
}
*/

void TLS3008::setOutputPin( byte bPin ) {
 this->_bLEDPin=bPin;
 pinMode(bPin, OUTPUT);
 digitalWrite(bPin, LOW);
}

void TLS3008::setPulseLength( byte bPulse ) {
	this->_bPulsUs = bPulse;
}

void TLS3008::setEOSyncDelay( byte bEOSD ) {
	this->_bEOSMs = bEOSD;
}

void TLS3008::setEOResetDelay( byte bEOR ) {
	this->_bEORMs = bEOR;
}

void TLS3008::setEOEmptyFrameDelay( byte bEOEF ) {
	this->_bEOEFrameMs = bEOEF;
}

void TLS3008::setEOFrameDelay( byte bEOF ) {
	this->_bEOFrameMs = bEOF;
}

void TLS3008::initCom() {
	reset();
	sync();
	reset();
	sync();
	reset();
	sync();
    interrupts(); // Interrupts ON
	sendEmptyFrame();
}

void TLS3008::sync()
 {
    noInterrupts(); // Interrupts OFF
	bitd(1);
    bitd(1);
    bitd(1);
    bitd(1);
    bitd(1);
    bitd(1);
    bitd(1);
    bitd(1);
    bitd(1);
    bitd(1);
    bitd(1);
    bitd(1);
    bitd(1);
    bitd(1);
    bitd(1);

    bitd(0);
    bitd(0);
    bitd(0);

    bitd(1);
    
    bitd(0);
    bitd(0);
    bitd(0);
    bitd(0);
    bitd(0);
    bitd(0);
    bitd(0);
    bitd(0);
    bitd(0);
    bitd(0);
    bitd(0);
    digitalWrite(this->_bLEDPin, LOW);

    interrupts(); // Interrupts ON
    delay(this->_bEOSMs);
 }

 void TLS3008::reset()
 {
	noInterrupts(); // Interrupts OFF
    bitd(1);
    bitd(1);
    bitd(1);
    bitd(1);
    bitd(1);
    bitd(1);
    bitd(1);
    bitd(1);
    bitd(1);
    bitd(1);
    bitd(1);
    bitd(1);
    bitd(1);
    bitd(1);
    bitd(1);

    bitd(0);

    bitd(1);

    bitd(0);
    bitd(0);
   digitalWrite(this->_bLEDPin, LOW);

   interrupts(); // Interrupts ON
   delay(this->_bEORMs);
 }

void TLS3008::frame_start()
 {
	noInterrupts(); // Interrupts OFF
    bitd(1);
    bitd(1);
    bitd(1);
    bitd(1);
    bitd(1);
    bitd(1);
    bitd(1);
    bitd(1);
    bitd(1);
    bitd(1);
    bitd(1);
    bitd(1);
    bitd(1);
    bitd(1);
    bitd(1);

    bitd(0);
    bitd(0);

    bitd(1);

    bitd(0);
 }

void TLS3008::bitd(boolean b)
 {
     digitalWrite(this->_bLEDPin, !!b);
     delayMicroseconds(this->_bPulsUs);
     digitalWrite(this->_bLEDPin, !b);
     delayMicroseconds(this->_bPulsUs-1);
 }

void TLS3008::sendEmptyFrame() {
       frame_start();
       digitalWrite(this->_bLEDPin, LOW);
       delay(this->_bEOEFrameMs);
       reset();
       sync();
	   this->_bFrameCount=0;
 }

 void TLS3008::sendRGBFrame(byte* ptRGB, unsigned int uiTableSize ) {
	unsigned int uiTS = uiTableSize;
	byte* ptTable = ptRGB;
	byte bMask;
	
	if (++(this->_bFrameCount)>MAXFRAMESBEFORESYNC) {
		this->_bFrameCount=0;
		sendEmptyFrame();
	}

	frame_start();
	
	while(uiTS > 0)
	{
		bitd(0); // Start tag
		for (bMask=0x80;bMask!=0;bMask>>=1) bitd( *ptTable & bMask );
		ptTable++;
		uiTS--;
	}
	digitalWrite(this->_bLEDPin, LOW);
	interrupts(); // Interrupts ON
	delay(this->_bEOEFrameMs);
 }