#ifndef TLS3008_H
#define TLS3008_H

// TLS3008 library Version 1.0.0
 
#if (ARDUINO >= 100)
  #include <Arduino.h>
#else
  #include <WProgram.h>
#endif

#define MAXFRAMESBEFORESYNC 20

class TLS3008 {
public:
        TLS3008( byte bPin );
//        ~TLS3008();
		// setOutputPin: Call this to setup the pin used for serial LED output. bPin is the pin number.
		void setOutputPin( byte bPin );
		// initCom: This MUST be called to initialize the communication with the LED string. Usually, this is done in the setup() function.
		void initCom();
		// sendEmptyFrame: Call this to send an empty DATA frame.
		void sendEmptyFrame(); // Private?
		// sendRGBFrame: Call this to send a DATA frame.
		// ptRGB is a pointer to a 'byte' array containing RGB data. Format of array is byte R, byte G, byte B.
		// uiTableSize is the size in bytes of the array.
		void sendRGBFrame( byte* ptRGB, unsigned int uiTableSize );

		// setPulseLength: Manchester half period length in microseconds
		void setPulseLength( byte bPulse );
		// setEOSyncDelay: Call this to change the default delay after sending a SYNC frame. Delay is in milliseconds.
		void setEOSyncDelay( byte bEOSD );
		// setEOResetDelay: Call this to change the default delay after sending a RESET frame. Delay is in milliseconds.
		void setEOResetDelay( byte bEOR );
		// setEOEmptyFrameDelay: Call this to change the default delay after sending an empty DATAHEADER frame. Delay is in milliseconds.
		void setEOEmptyFrameDelay( byte bEOEF );
		// setEOEmptyFrameDelay: Call this to change the default delay after sending a DATA frame. Delay is in milliseconds.
		void setEOFrameDelay( byte bEOF );
private:
		byte _bLEDPin;
		byte _bPulsUs; // Pulse length
		byte _bEOSMs; // End Of Sync delay in ms
		byte _bEORMs; // End Of Reset delay in ms
		byte _bEOEFrameMs; // End of Empty Data Frame delay in ms
		byte _bEOFrameMs; // End of Data Frame delay (end of all leds data in string)
		byte _bFrameCount; // Sent Frame counter

		void bitd(boolean b);
		void sync();
		void reset();
		void frame_start();
};

#endif